@@ -14,6 +14,7 @@
 
 #include <limits>
 
+#include "reading_list.h"
 #include "library_item.h"
 #include "comic_db.h"
 #include "data_base_management.h"
@@ -39,7 +40,7 @@ QList<LibraryItem *> DBHelper::getFolderSubfoldersFromLibrary(qulonglong library
 	QList<LibraryItem *> list = DBHelper::getFoldersFromParent(folderId,db,false);
 	
 	db.close();
-	QSqlDatabase::removeDatabase(libraryPath);
+	QSqlDatabase::removeDatabase(db.connectionName());
 	return list;
 }
 QList<LibraryItem *> DBHelper::getFolderComicsFromLibrary(qulonglong libraryId, qulonglong folderId)
@@ -55,9 +56,41 @@ QList<LibraryItem *> DBHelper::getFolderComicsFromLibrary(qulonglong libraryId,
     QList<LibraryItem *> list = DBHelper::getComicsFromParent(folderId,db,sort);
 
     db.close();
-    QSqlDatabase::removeDatabase(libraryPath);
+    QSqlDatabase::removeDatabase(db.connectionName());
     return list;
 }
+
+quint32 DBHelper::getNumChildrenFromFolder(qulonglong libraryId, qulonglong folderId)
+{
+    QString libraryPath = DBHelper::getLibraries().getPath(libraryId);
+    QSqlDatabase db = DataBaseManagement::loadDatabase(libraryPath+"/.yacreaderlibrary");
+
+    quint32 result = 0;
+
+    {
+        QSqlQuery selectQuery(db);
+        selectQuery.prepare("SELECT count(*) FROM folder WHERE parentId = :parentId and id <> 1");
+        selectQuery.bindValue(":parentId", folderId);
+        selectQuery.exec();
+
+        result +=  selectQuery.record().value(0).toULongLong();
+    }
+
+    {
+        QSqlQuery selectQuery(db);
+        selectQuery.prepare("SELECT count(*) FROM comic c WHERE c.parentId = :parentId");
+        selectQuery.bindValue(":parentId", folderId);
+        selectQuery.exec();
+
+        result +=  selectQuery.record().value(0).toULongLong();
+    }
+
+    db.close();
+    QSqlDatabase::removeDatabase(db.connectionName());
+
+    return result;
+}
+
 qulonglong DBHelper::getParentFromComicFolderId(qulonglong libraryId, qulonglong id)
 {
     QString libraryPath = DBHelper::getLibraries().getPath(libraryId);
@@ -66,7 +99,7 @@ qulonglong DBHelper::getParentFromComicFolderId(qulonglong libraryId, qulonglong
 	Folder f = DBHelper::loadFolder(id,db);
 
 	db.close();
-	QSqlDatabase::removeDatabase(libraryPath);
+	QSqlDatabase::removeDatabase(db.connectionName());
 	return f.parentId;
 }
 ComicDB DBHelper::getComicInfo(qulonglong libraryId, qulonglong id)
@@ -77,7 +110,7 @@ ComicDB DBHelper::getComicInfo(qulonglong libraryId, qulonglong id)
 	ComicDB comic = DBHelper::loadComic(id,db);
 
 	db.close();
-	QSqlDatabase::removeDatabase(libraryPath);
+	QSqlDatabase::removeDatabase(db.connectionName());
 	return comic;
 }
 
@@ -88,7 +121,7 @@ QList<ComicDB> DBHelper::getSiblings(qulonglong libraryId, qulonglong parentId)
 
 	QList<ComicDB> comics =  DBHelper::getSortedComicsFromParent(parentId,db);
 	db.close();
-	QSqlDatabase::removeDatabase(libraryPath);
+	QSqlDatabase::removeDatabase(db.connectionName());
 	return comics;
 }
 
@@ -107,13 +140,12 @@ QString DBHelper::getFolderName(qulonglong libraryId, qulonglong id)
 
 		if(selectQuery.next()) 
 		{
-			QSqlRecord record = selectQuery.record();
-			name = record.value(0).toString();
+            name = selectQuery.value(0).toString();
 		}
 	}
 
 	db.close();
-	QSqlDatabase::removeDatabase(libraryPath);
+	QSqlDatabase::removeDatabase(db.connectionName());
 	return name;
 }
 QList<QString> DBHelper::getLibrariesNames()
@@ -126,14 +158,238 @@ QString DBHelper::getLibraryName(int id)
 {
 	return getLibraries().getName(id);
 }
+
+QList<ComicDB> DBHelper::getLabelComics(qulonglong libraryId, qulonglong labelId)
+{
+    QString libraryPath = DBHelper::getLibraries().getPath(libraryId);
+    QSqlDatabase db = DataBaseManagement::loadDatabase(libraryPath+"/.yacreaderlibrary");
+
+    QList<ComicDB> list;
+
+    {
+        QSqlQuery selectQuery(db);
+        selectQuery.prepare("SELECT c.id,c.fileName,ci.title,ci.currentPage,ci.numPages,ci.hash,ci.read,ci.coverSizeRatio "
+                            "FROM comic c INNER JOIN comic_info ci ON (c.comicInfoId = ci.id) "
+                            "INNER JOIN comic_label cl ON (c.id == cl.comic_id) "
+                            "WHERE cl.label_id = :parentLabelId "
+                            "ORDER BY cl.ordering");
+        selectQuery.bindValue(":parentLabelId", labelId);
+        selectQuery.exec();
+
+        while (selectQuery.next())
+        {
+            ComicDB comic;
+
+            comic.id = selectQuery.value(0).toULongLong();
+            comic.parentId = labelId;
+            comic.name = selectQuery.value(1).toString();
+            comic.info.title = selectQuery.value(2).toString();
+            comic.info.currentPage = selectQuery.value(3).toInt();
+            comic.info.numPages = selectQuery.value(4).toInt();
+            comic.info.hash = selectQuery.value(5).toString();
+            comic.info.read = selectQuery.value(6).toBool();
+            comic.info.coverSizeRatio = selectQuery.value(7).toFloat();
+
+            list.append(comic);
+        }
+
+        db.close();
+    }
+    //TODO ?
+    //QSqlDatabase::removeDatabase(db.connectionName());
+
+    return list;
+}
+
+QList<ComicDB> DBHelper::getFavorites(qulonglong libraryId)
+{
+    QString libraryPath = DBHelper::getLibraries().getPath(libraryId);
+    QSqlDatabase db = DataBaseManagement::loadDatabase(libraryPath+"/.yacreaderlibrary");
+
+    QList<ComicDB> list;
+
+    const int FAV_ID = 1;
+
+    {
+        QSqlQuery selectQuery(db);
+        selectQuery.prepare("SELECT c.id,c.fileName,ci.title,ci.currentPage,ci.numPages,ci.hash,ci.read,ci.coverSizeRatio "
+                            "FROM comic c INNER JOIN comic_info ci ON (c.comicInfoId = ci.id) "
+                            "INNER JOIN comic_default_reading_list cdrl ON (c.id == cdrl.comic_id) "
+                            "WHERE cdrl.default_reading_list_id = :parentDefaultListId "
+                            "ORDER BY cdrl.ordering");
+        selectQuery.bindValue(":parentDefaultListId", FAV_ID);
+        selectQuery.exec();
+
+        while (selectQuery.next())
+        {
+            ComicDB comic;
+
+            comic.id = selectQuery.value(0).toULongLong();
+            comic.parentId = FAV_ID;
+            comic.name = selectQuery.value(1).toString();
+            comic.info.title = selectQuery.value(2).toString();
+            comic.info.currentPage = selectQuery.value(3).toInt();
+            comic.info.numPages = selectQuery.value(4).toInt();
+            comic.info.hash = selectQuery.value(5).toString();
+            comic.info.read = selectQuery.value(6).toBool();
+            comic.info.coverSizeRatio = selectQuery.value(7).toFloat();
+
+            list.append(comic);
+        }
+
+        db.close();
+    }
+    //TODO ?
+    //QSqlDatabase::removeDatabase(db.connectionName());
+    
+    return list;
+}
+
+QList<ComicDB> DBHelper::getReading(qulonglong libraryId)
+{
+    QString libraryPath = DBHelper::getLibraries().getPath(libraryId);
+    QSqlDatabase db = DataBaseManagement::loadDatabase(libraryPath+"/.yacreaderlibrary");
+
+    QList<ComicDB> list;
+
+    {
+        QSqlQuery selectQuery(db);
+        selectQuery.prepare("SELECT c.id,c.parentId,c.fileName,ci.title,ci.currentPage,ci.numPages,ci.hash,ci.read,ci.coverSizeRatio "
+                            "FROM comic c INNER JOIN comic_info ci ON (c.comicInfoId = ci.id) "
+                            "WHERE ci.hasBeenOpened = 1 AND ci.read = 0 "
+                            "ORDER BY ci.lastTimeOpened DESC");
+        selectQuery.exec();
+
+        while (selectQuery.next())
+        {
+            ComicDB comic;
+
+            comic.id = selectQuery.value(0).toULongLong();
+            comic.parentId = selectQuery.value(1).toULongLong();
+            comic.name = selectQuery.value(2).toString();
+            comic.info.title = selectQuery.value(3).toString();
+            comic.info.currentPage = selectQuery.value(4).toInt();
+            comic.info.numPages = selectQuery.value(5).toInt();
+            comic.info.hash = selectQuery.value(6).toString();
+            comic.info.read = selectQuery.value(7).toBool();
+            comic.info.coverSizeRatio = selectQuery.value(8).toFloat();
+
+            list.append(comic);
+        }
+
+        db.close();
+    }
+    //TODO ?
+    //QSqlDatabase::removeDatabase(db.connectionName());
+    
+    return list;
+}
+
+QList<ReadingList> DBHelper::getReadingLists(qulonglong libraryId)
+{
+    QString libraryPath = DBHelper::getLibraries().getPath(libraryId);
+    QSqlDatabase db = DataBaseManagement::loadDatabase(libraryPath+"/.yacreaderlibrary");
+
+    QList<ReadingList> list;
+
+    QSqlQuery selectQuery("SELECT * from reading_list WHERE parentId IS NULL ORDER BY name DESC",db);
+
+    selectQuery.exec();
+
+    QSqlRecord record = selectQuery.record();
+
+    int name = record.indexOf("name");
+    int id = record.indexOf("id");
+    int ordering = record.indexOf("ordering");
+
+    while (selectQuery.next())
+    {
+        ReadingList item(selectQuery.value(name).toString(), selectQuery.value(id).toLongLong(),selectQuery.value(ordering).toInt());
+
+        if(list.isEmpty())
+        {
+            list.append(item);
+        }
+        else
+        {
+            int i= 0;
+            while(i<list.length() && naturalSortLessThanCI(list.at(i).getName(),item.getName()))
+                i++;
+            list.insert(i,item);
+        }
+    }
+
+    //TODO ?
+    //QSqlDatabase::removeDatabase(db.connectionName());
+    
+    return list;
+}
+
+QList<ComicDB> DBHelper::getReadingListFullContent(qulonglong libraryId, qulonglong readingListId)
+{
+    QString libraryPath = DBHelper::getLibraries().getPath(libraryId);
+    QSqlDatabase db = DataBaseManagement::loadDatabase(libraryPath+"/.yacreaderlibrary");
+
+    QList<ComicDB> list;
+
+    {
+        QList<qulonglong> ids;
+        ids << readingListId;
+
+        QSqlQuery subfolders(db);
+        subfolders.prepare("SELECT id "
+                           "FROM reading_list "
+                           "WHERE parentId = :parentId "
+                           "ORDER BY ordering ASC");
+        subfolders.bindValue(":parentId", readingListId);
+        subfolders.exec();
+        while(subfolders.next())
+           ids << subfolders.value(0).toULongLong();
+
+        foreach(qulonglong id, ids)
+        {
+            QSqlQuery selectQuery(db);
+            selectQuery.prepare("SELECT c.id,c.parentId,c.fileName,ci.title,ci.currentPage,ci.numPages,ci.hash,ci.read,ci.coverSizeRatio "
+                                "FROM comic c INNER JOIN comic_info ci ON (c.comicInfoId = ci.id) "
+                                "INNER JOIN comic_reading_list crl ON (c.id == crl.comic_id) "
+                                "WHERE crl.reading_list_id = :parentReadingList "
+                                "ORDER BY crl.ordering");
+            selectQuery.bindValue(":parentReadingList", id);
+            selectQuery.exec();
+
+            while (selectQuery.next())
+            {
+                ComicDB comic;
+
+                comic.id = selectQuery.value(0).toULongLong();
+                comic.parentId = selectQuery.value(1).toULongLong();
+                comic.name = selectQuery.value(2).toString();
+                comic.info.title = selectQuery.value(3).toString();
+                comic.info.currentPage = selectQuery.value(4).toInt();
+                comic.info.numPages = selectQuery.value(5).toInt();
+                comic.info.hash = selectQuery.value(6).toString();
+                comic.info.read = selectQuery.value(7).toBool();
+                comic.info.coverSizeRatio = selectQuery.value(8).toFloat();
+
+                list.append(comic);
+            }
+        }
+    }
+    
+    //TODO ?
+    //QSqlDatabase::removeDatabase(db.connectionName());
+    
+    return list;
+}
+
 //objects management
 //deletes
 void DBHelper::removeFromDB(LibraryItem * item, QSqlDatabase & db)
 {
 	if(item->isDir())
-		DBHelper::removeFromDB(dynamic_cast<Folder *>(item),db);
+        DBHelper::removeFromDB(dynamic_cast<Folder *>(item),db);
 	else
-		DBHelper::removeFromDB(dynamic_cast<ComicDB *>(item),db);
+        DBHelper::removeFromDB(dynamic_cast<ComicDB *>(item),db);
 }
 void DBHelper::removeFromDB(Folder * folder, QSqlDatabase & db)
 {
@@ -238,7 +494,7 @@ void DBHelper::update(qulonglong libraryId, ComicInfo & comicInfo)
 	DBHelper::update(&comicInfo,db);
 
 	db.close();
-	QSqlDatabase::removeDatabase(libraryPath);
+	QSqlDatabase::removeDatabase(db.connectionName());
 }
 
 void DBHelper::update(ComicInfo * comicInfo, QSqlDatabase & db)
@@ -296,7 +552,13 @@ void DBHelper::update(ComicInfo * comicInfo, QSqlDatabase & db)
         "rating = :rating,"
 
         //new 7.1 fields
-        "comicVineID = :comicVineID"
+        "comicVineID = :comicVineID,"
+
+        //new 9.5 fields
+        "lastTimeOpened = :lastTimeOpened,"
+
+        "coverSizeRatio = :coverSizeRatio,"
+        "originalCoverSize = :originalCoverSize"
 		//--
 		" WHERE id = :id ");
 
@@ -339,7 +601,7 @@ void DBHelper::update(ComicInfo * comicInfo, QSqlDatabase & db)
 	updateComicInfo.bindValue(":id", comicInfo->id);
 	updateComicInfo.bindValue(":edited", comicInfo->edited?1:0);
 
-	updateComicInfo.bindValue(":hasBeenOpened", comicInfo->hasBeenOpened?1:0);
+    updateComicInfo.bindValue(":hasBeenOpened", comicInfo->hasBeenOpened?1:0 || comicInfo->currentPage > 1);
 	updateComicInfo.bindValue(":currentPage", comicInfo->currentPage);
 	updateComicInfo.bindValue(":bookmark1", comicInfo->bookmark1);
 	updateComicInfo.bindValue(":bookmark2", comicInfo->bookmark2);
@@ -351,6 +613,11 @@ void DBHelper::update(ComicInfo * comicInfo, QSqlDatabase & db)
 
     updateComicInfo.bindValue(":comicVineID", comicInfo->comicVineID);
 
+    updateComicInfo.bindValue(":lastTimeOpened", comicInfo->lastTimeOpened);
+
+    updateComicInfo.bindValue(":coverSizeRatio", comicInfo->coverSizeRatio);
+    updateComicInfo.bindValue(":originalCoverSize", comicInfo->originalCoverSize);
+
     updateComicInfo.exec();
 }
 
@@ -379,6 +646,51 @@ void DBHelper::update(const Folder & folder, QSqlDatabase &db)
     updateFolderInfo.exec();
 }
 
+void DBHelper::updateChildrenInfo(const Folder & folder, QSqlDatabase & db)
+{
+    QSqlQuery updateFolderInfo(db);
+    updateFolderInfo.prepare("UPDATE folder SET "
+                             "numChildren = :numChildren, "
+                             "firstChildHash = :firstChildHash "
+                             "WHERE id = :id ");
+    updateFolderInfo.bindValue(":numChildren", folder.getNumChildren());
+    updateFolderInfo.bindValue(":firstChildHash", folder.getFirstChildHash());
+    updateFolderInfo.bindValue(":id", folder.id);
+    updateFolderInfo.exec();
+}
+
+void DBHelper::updateChildrenInfo(qulonglong folderId, QSqlDatabase & db)
+{
+    QList<LibraryItem *> subfolders = DBHelper::getFoldersFromParent(folderId,db,false);
+    QList<LibraryItem *> comics = DBHelper::getComicsFromParent(folderId,db,true);
+
+    ComicDB * firstComic = NULL;
+    if(comics.count() > 0)
+        firstComic = static_cast<ComicDB *>(comics.first());
+
+    QSqlQuery updateFolderInfo(db);
+    updateFolderInfo.prepare("UPDATE folder SET "
+                             "numChildren = :numChildren, "
+                             "firstChildHash = :firstChildHash "
+                             "WHERE id = :id ");
+    updateFolderInfo.bindValue(":numChildren", subfolders.count() + comics.count());
+    updateFolderInfo.bindValue(":firstChildHash", firstComic != NULL ? firstComic->info.hash : "");
+    updateFolderInfo.bindValue(":id", folderId);
+    updateFolderInfo.exec();
+}
+
+void DBHelper::updateChildrenInfo(QSqlDatabase & db)
+{
+    QSqlQuery selectQuery(db); //TODO check
+    selectQuery.prepare("SELECT id FROM folder");
+    selectQuery.exec();
+
+    while (selectQuery.next())
+    {
+        DBHelper::updateChildrenInfo(selectQuery.value(0).toULongLong(), db);
+    }
+}
+
 void DBHelper::updateProgress(qulonglong libraryId, const ComicInfo &comicInfo)
 {
     QString libraryPath = DBHelper::getLibraries().getPath(libraryId);
@@ -386,13 +698,28 @@ void DBHelper::updateProgress(qulonglong libraryId, const ComicInfo &comicInfo)
 
     ComicDB comic = DBHelper::loadComic(comicInfo.id,db);
     comic.info.currentPage = comicInfo.currentPage;
+    comic.info.hasBeenOpened = comicInfo.currentPage > 0 || comic.info.hasBeenOpened;
+    comic.info.read = comic.info.read || comic.info.currentPage == comic.info.numPages;
+
+    DBHelper::updateReadingRemoteProgress(comic.info,db);
+
+    db.close();
+    QSqlDatabase::removeDatabase(db.connectionName());
+}
+
+void DBHelper::setComicAsReading(qulonglong libraryId, const ComicInfo &comicInfo)
+{
+    QString libraryPath = DBHelper::getLibraries().getPath(libraryId);
+    QSqlDatabase db = DataBaseManagement::loadDatabase(libraryPath+"/.yacreaderlibrary");
+
+    ComicDB comic = DBHelper::loadComic(comicInfo.id,db);
     comic.info.hasBeenOpened = true;
     comic.info.read = comic.info.read || comic.info.currentPage == comic.info.numPages;
 
     DBHelper::updateReadingRemoteProgress(comic.info,db);
 
     db.close();
-    QSqlDatabase::removeDatabase(libraryPath);
+    QSqlDatabase::removeDatabase(db.connectionName());
 }
 
 void DBHelper::updateReadingRemoteProgress(const ComicInfo &comicInfo, QSqlDatabase &db)
@@ -402,12 +729,14 @@ void DBHelper::updateReadingRemoteProgress(const ComicInfo &comicInfo, QSqlDatab
                             "read = :read, "
                             "currentPage = :currentPage, "
                             "hasBeenOpened = :hasBeenOpened, "
+                            "lastTimeOpened = :lastTimeOpened, "
                             "rating = :rating"
                             " WHERE id = :id ");
 
     updateComicInfo.bindValue(":read", comicInfo.read?1:0);
     updateComicInfo.bindValue(":currentPage", comicInfo.currentPage);
     updateComicInfo.bindValue(":hasBeenOpened", comicInfo.hasBeenOpened?1:0);
+    updateComicInfo.bindValue(":lastTimeOpened", QDateTime::currentMSecsSinceEpoch() / 1000);
     updateComicInfo.bindValue(":id", comicInfo.id);
     updateComicInfo.bindValue(":rating", comicInfo.rating);
     updateComicInfo.exec();
@@ -427,12 +756,15 @@ void DBHelper::updateFromRemoteClient(qulonglong libraryId,const ComicInfo & com
     {
         if(comicInfo.currentPage > 0)
         {
+            comic.info.currentPage = comicInfo.currentPage;
+
             if(comic.info.currentPage == comic.info.numPages)
                 comic.info.read = true;
 
-            comic.info.currentPage = comicInfo.currentPage;
-
             comic.info.hasBeenOpened = true;
+
+            if (comic.info.lastTimeOpened.toULongLong() < comicInfo.lastTimeOpened.toULongLong())
+                comic.info.lastTimeOpened = comicInfo.lastTimeOpened;
         }
 
         if(comicInfo.rating > 0)
@@ -442,7 +774,43 @@ void DBHelper::updateFromRemoteClient(qulonglong libraryId,const ComicInfo & com
     }
 
     db.close();
-    QSqlDatabase::removeDatabase(libraryPath);
+    QSqlDatabase::removeDatabase(db.connectionName());
+}
+
+void DBHelper::updateFromRemoteClientWithHash(const ComicInfo & comicInfo)
+{
+     YACReaderLibraries libraries = DBHelper::getLibraries();
+
+     QStringList names = libraries.getNames();
+
+     foreach (QString name, names) {
+         QString libraryPath = DBHelper::getLibraries().getPath(libraries.getId(name));
+
+         QSqlDatabase db = DataBaseManagement::loadDatabase(libraryPath+"/.yacreaderlibrary");
+
+         ComicInfo info = loadComicInfo(comicInfo.hash, db);
+
+         if(comicInfo.currentPage > 0)
+         {
+             info.currentPage = comicInfo.currentPage;
+
+             if(info.currentPage == info.numPages)
+                 info.read = true;
+
+             info.hasBeenOpened = true;
+
+             if (info.lastTimeOpened.toULongLong() < comicInfo.lastTimeOpened.toULongLong())
+                 info.lastTimeOpened = comicInfo.lastTimeOpened;
+         }
+
+         if(comicInfo.rating > 0)
+             info.rating = comicInfo.rating;
+
+         DBHelper::update(&info, db);
+
+         db.close();
+         QSqlDatabase::removeDatabase(db.connectionName());
+     }
 }
 
 void DBHelper::renameLabel(qulonglong id, const QString &name, QSqlDatabase &db)
@@ -492,7 +860,7 @@ void DBHelper::reasignOrderToComicsInFavorites(QList<qulonglong> comicIds, QSqlD
     QSqlQuery updateOrdering(db);
     updateOrdering.prepare("UPDATE comic_default_reading_list SET "
                            "ordering = :ordering "
-                           "WHERE comic_id = :comic_id AND default_reading_list_id = 0");
+                           "WHERE comic_id = :comic_id AND default_reading_list_id = 1");
     db.transaction();
     int order = 0;
     foreach(qulonglong id, comicIds)
@@ -554,6 +922,7 @@ qulonglong DBHelper::insert(Folder * folder, QSqlDatabase & db)
 	query.bindValue(":name", folder->name);
 	query.bindValue(":path", folder->path);
 	query.exec();
+
 	return query.lastInsertId().toULongLong();
 }
 
@@ -562,10 +931,12 @@ qulonglong DBHelper::insert(ComicDB * comic, QSqlDatabase & db)
 	if(!comic->info.existOnDb)
 	{
 		QSqlQuery comicInfoInsert(db);
-		comicInfoInsert.prepare("INSERT INTO comic_info (hash,numPages) "
-			"VALUES (:hash,:numPages)");
+        comicInfoInsert.prepare("INSERT INTO comic_info (hash,numPages,coverSizeRatio,originalCoverSize) "
+            "VALUES (:hash,:numPages,:coverSizeRatio,:originalCoverSize)");
 		comicInfoInsert.bindValue(":hash", comic->info.hash);
         comicInfoInsert.bindValue(":numPages", comic->info.numPages);
+        comicInfoInsert.bindValue(":coverSizeRatio", comic->info.coverSizeRatio);
+        comicInfoInsert.bindValue(":originalCoverSize", comic->info.originalCoverSize);
 		comicInfoInsert.exec();
 		comic->info.id =comicInfoInsert.lastInsertId().toULongLong();
 		comic->_hasCover = false;
@@ -581,6 +952,7 @@ qulonglong DBHelper::insert(ComicDB * comic, QSqlDatabase & db)
     query.bindValue(":name", comic->name);
     query.bindValue(":path", comic->path);
 	query.exec();
+
     return query.lastInsertId().toULongLong();
 }
 
@@ -638,6 +1010,8 @@ void DBHelper::insertComicsInFavorites(const QList<ComicDB> &comicsList, QSqlDat
         query.exec();
     }
 
+    QLOG_TRACE() << query.lastError();
+    
     db.commit();
 }
 
@@ -662,6 +1036,8 @@ void DBHelper::insertComicsInLabel(const QList<ComicDB> &comicsList, qulonglong
         query.exec();
     }
 
+    QLOG_TRACE() << query.lastError();
+    
     db.commit();
 }
 
@@ -703,13 +1079,22 @@ QList<LibraryItem *> DBHelper::getFoldersFromParent(qulonglong parentId, QSqlDat
     int name = record.indexOf("name");
     int path = record.indexOf("path");
     int id = record.indexOf("id");
+    int numChildren = record.indexOf("numChildren");
+    int firstChildHash = record.indexOf("firstChildHash");
+    int customImage = record.indexOf("customImage");
 
     Folder * currentItem;
 	while (selectQuery.next()) 
 	{
 		//TODO sort by sort indicator and name
-        currentItem = new Folder(selectQuery.value(id).toULongLong(),parentId,selectQuery.value(name).toString(),selectQuery.value(path).toString());
-		int lessThan = 0;
+		currentItem = new Folder(selectQuery.value(id).toULongLong(),parentId,selectQuery.value(name).toString(),selectQuery.value(path).toString());
+
+        if(!selectQuery.value(numChildren).isNull() && selectQuery.value(numChildren).isValid())
+             currentItem->setNumChildren(selectQuery.value(numChildren).toInt());
+        currentItem->setFirstChildHash(selectQuery.value(firstChildHash).toString());
+        currentItem->setCustomImage(selectQuery.value(customImage).toString());
+
+        int lessThan = 0;
 
 		if(list.isEmpty() || !sort)
 			list.append(currentItem);
@@ -721,7 +1106,7 @@ QList<LibraryItem *> DBHelper::getFoldersFromParent(qulonglong parentId, QSqlDat
 			QList<LibraryItem *>::iterator i;
 			i = list.end();
 			i--;
-			while ((0 > (lessThan = naturalSortLessThanCI(nameCurrent,nameLast))) && i != list.begin())
+            while ((0 > (lessThan = naturalCompare(nameCurrent,nameLast,Qt::CaseInsensitive))) && i != list.begin())
 			{
 				i--;
 				nameLast = (*i)->name;
@@ -806,6 +1191,11 @@ QList<ComicDB> DBHelper::getSortedComicsFromParent(qulonglong parentId, QSqlData
 
     int comicVineID = record.indexOf("comicVineID");
 
+    int lastTimeOpened = record.indexOf("lastTimeOpened");
+
+    int coverSizeRatio = record.indexOf("coverSizeRatio");
+    int originalCoverSize = record.indexOf("originalCoverSize");
+
 	ComicDB currentItem;
 	while (selectQuery.next()) 
 	{
@@ -866,6 +1256,11 @@ QList<ComicDB> DBHelper::getSortedComicsFromParent(qulonglong parentId, QSqlData
 
         currentItem.info.comicVineID = selectQuery.value(comicVineID);
 
+        currentItem.info.lastTimeOpened = selectQuery.value(lastTimeOpened);
+
+        currentItem.info.coverSizeRatio = selectQuery.value(coverSizeRatio);
+        currentItem.info.originalCoverSize = selectQuery.value(originalCoverSize);
+
         currentItem.info.existOnDb = true;
 
         list.append(currentItem);
@@ -929,6 +1324,63 @@ QList<LibraryItem *> DBHelper::getComicsFromParent(qulonglong parentId, QSqlData
 	return list;
 }
 
+QList<Label> DBHelper::getLabels(qulonglong libraryId)
+{
+    QString libraryPath = DBHelper::getLibraries().getPath(libraryId);
+    QSqlDatabase db = DataBaseManagement::loadDatabase(libraryPath+"/.yacreaderlibrary");
+
+    QSqlQuery selectQuery("SELECT * FROM label ORDER BY ordering,name",db); //TODO add some kind of
+    QList<Label> labels;
+
+    QSqlRecord record = selectQuery.record();
+
+    int name =  record.indexOf("name");
+    int color = record.indexOf("color");
+    int id = record.indexOf("id");
+    int ordering = record.indexOf("ordering");
+
+    while(selectQuery.next())
+    {
+        Label item(selectQuery.value(name).toString(),
+                   selectQuery.value(id).toLongLong(),
+                   static_cast<YACReader::LabelColors>(selectQuery.value(ordering).toInt()));
+
+        if(labels.isEmpty())
+        {
+            labels << item;
+        }
+        else
+        {
+            int i = 0;
+
+            while (i < labels.count() && (labels.at(i).getColorID() < item.getColorID()) )
+                i++;
+
+            if(i < labels.count())
+            {
+                if(labels.at(i).getColorID() == item.getColorID()) //sort by name
+                {
+                    while( i < labels.count() && labels.at(i).getColorID() == item.getColorID() && naturalSortLessThanCI(labels.at(i).getName(),item.getName()))
+                        i++;
+                }
+            }
+            if(i >= labels.count())
+            {
+                labels << item;
+            }
+            else
+            {
+                labels.insert(i,item);
+            }
+        }
+    }
+
+    db.close();
+    QSqlDatabase::removeDatabase(db.connectionName());
+
+    return labels;
+}
+
 //loads
 Folder DBHelper::loadFolder(qulonglong id, QSqlDatabase & db)
 {
@@ -948,6 +1400,9 @@ Folder DBHelper::loadFolder(qulonglong id, QSqlDatabase & db)
     int path = record.indexOf("path");
     int finished = record.indexOf("finished");
     int completed = record.indexOf("completed");
+    int numChildren = record.indexOf("numChildren");
+    int firstChildHash = record.indexOf("firstChildHash");
+    int customImage = record.indexOf("customImage");
 
 	if(query.next())
 	{
@@ -955,9 +1410,16 @@ Folder DBHelper::loadFolder(qulonglong id, QSqlDatabase & db)
         folder.name = query.value(name).toString();
         folder.path = query.value(path).toString();
         folder.knownId = true;
+
         //new 7.1
         folder.setFinished(query.value(finished).toBool());
         folder.setCompleted(query.value(completed).toBool());
+
+        //new 9.5
+        if(!query.value(numChildren).isNull() && query.value(numChildren).isValid())
+             folder.setNumChildren(query.value(numChildren).toInt());
+        folder.setFirstChildHash(query.value(firstChildHash).toString());
+        folder.setCustomImage(query.value(customImage).toString());
 	}
 
     return folder;
@@ -980,6 +1442,9 @@ Folder DBHelper::loadFolder(const QString &folderName, qulonglong parentId, QSql
     int path = record.indexOf("path");
     int finished = record.indexOf("finished");
     int completed = record.indexOf("completed");
+    int numChildren = record.indexOf("numChildren");
+    int firstChildHash = record.indexOf("firstChildHash");
+    int customImage = record.indexOf("customImage");
 
     folder.parentId = parentId;
     if(query.next())
@@ -988,9 +1453,16 @@ Folder DBHelper::loadFolder(const QString &folderName, qulonglong parentId, QSql
         folder.name = query.value(name).toString();
         folder.path = query.value(path).toString();
         folder.knownId = true;
+
         //new 7.1
         folder.setFinished(query.value(finished).toBool());
         folder.setCompleted(query.value(completed).toBool());
+
+        //new 9.5
+        if(!query.value(numChildren).isNull() && query.value(numChildren).isValid())
+             folder.setNumChildren(query.value(numChildren).toInt());
+        folder.setFirstChildHash(query.value(firstChildHash).toString());
+        folder.setCustomImage(query.value(customImage).toString());
     }
 
     return folder;
@@ -1008,7 +1480,7 @@ ComicDB DBHelper::loadComic(qulonglong id, QSqlDatabase & db)
     QSqlRecord record = selectQuery.record();
 
     int parentId = record.indexOf("parentId");
-    int name = record.indexOf("name");
+    int name = record.indexOf("fileName");
     int path = record.indexOf("path");
     int hash = record.indexOf("hash");
 
@@ -1108,6 +1580,11 @@ ComicInfo DBHelper::loadComicInfo(QString hash, QSqlDatabase & db)
 
     int comicVineID = record.indexOf("comicVineID");
 
+    int lastTimeOpened = record.indexOf("lastTimeOpened");
+
+    int coverSizeRatio = record.indexOf("coverSizeRatio");
+    int originalCoverSize = record.indexOf("originalCoverSize");
+
 	if(findComicInfo.next())
 	{
 		comicInfo.hash = hash;
@@ -1126,7 +1603,6 @@ ComicInfo DBHelper::loadComicInfo(QString hash, QSqlDatabase & db)
         comicInfo.gamma = findComicInfo.value(gamma).toInt();
         comicInfo.rating = findComicInfo.value(rating).toInt();
 		//--
-
         comicInfo.title = findComicInfo.value(title);
         comicInfo.numPages = findComicInfo.value(numPages);
 
@@ -1162,6 +1638,13 @@ ComicInfo DBHelper::loadComicInfo(QString hash, QSqlDatabase & db)
 
         comicInfo.comicVineID = findComicInfo.value(comicVineID);
 
+        //new 9.5 fields
+        comicInfo.lastTimeOpened = findComicInfo.value(lastTimeOpened);
+
+        comicInfo.coverSizeRatio = findComicInfo.value(coverSizeRatio);
+        comicInfo.originalCoverSize = findComicInfo.value(originalCoverSize);
+        //--
+
 		comicInfo.existOnDb = true;
 	}
 	else
