@@ -83,6 +83,10 @@ public:
 
 	QImage cover;
 
+    QVariant lastTimeOpened;//integer/date
+    QVariant coverSizeRatio;//h/w
+    QVariant originalCoverSize; //string "WxH"
+
     /*void setTitle(QVariant value);
 
     void setCoverPage(QVariant value);
@@ -186,6 +190,11 @@ public:
 
     Q_PROPERTY(QImage cover MEMBER cover CONSTANT)
 
+    Q_PROPERTY(QVariant lastTimeOpened MEMBER lastTimeOpened CONSTANT)
+
+    Q_PROPERTY(QVariant coverSizeRatio MEMBER coverSizeRatio CONSTANT)
+    Q_PROPERTY(QVariant originalCoverSize MEMBER originalCoverSize CONSTANT)
+
     //-new properties, not loaded from the DB automatically
     bool isFavorite;
     Q_PROPERTY(bool isFavorite MEMBER isFavorite WRITE setFavorite NOTIFY favoriteChanged)
@@ -210,7 +219,7 @@ public:
 	ComicDB();
     ComicDB(const ComicDB & comicDB);
 	
-	bool isDir();
+    bool isDir() const;
 	
 	bool _hasCover;
 
